#include "ehs/io/socket/BaseICMP.h"
#include "ehs/Serializer.h"

namespace ehs
{


	BaseICMP::BaseICMP()
		: version(IP::V6)
	{
	}

	BaseICMP::BaseICMP(const IP version)
		: version(version)
	{
	}

	BaseICMP::BaseICMP(BaseICMP &&icmp) noexcept
		: version(icmp.version)
	{
	}

	BaseICMP::BaseICMP(const BaseICMP &icmp)
		: version(icmp.version)
	{
	}

	BaseICMP & BaseICMP::operator=(BaseICMP &&icmp) noexcept
	{
		if (this == &icmp)
			return *this;

		version = icmp.version;

		return *this;
	}

	BaseICMP & BaseICMP::operator=(const BaseICMP &icmp)
	{
		if (this == &icmp)
			return *this;

		version = icmp.version;

		return *this;
	}

	void BaseICMP::Release()
	{
	}

	UInt_64 BaseICMP::Send(const Str_8 &address, ICMP_Header header, const Byte *data, UInt_64 size)
	{
		if (GetVersion() == IP::V6)
			return SendV6(address, header, data, size);

		return SendV4(address, header, data, size);
	}

	UInt_64 BaseICMP::Receive(Str_8 &address, ICMP_Header &header, Serializer<UInt_64> &data)
	{
		if (GetVersion() == IP::V6)
			return ReceiveV6(address, header, data);

		return ReceiveV4(address, header, data);
	}

	void BaseICMP::SendEchoRequest(const Str_8 &address, ICMP_EchoRequest er, const Byte *data, UInt_64 size)
	{
		if (!IsValid())
		{
			EHS_LOG_INT(LogType::WARN, 0, "Socket is not initialized.");

			return;
		}

		ICMP_Header header = {
			version == IP::V6 ? (UInt_8)128 : (UInt_8)8,
			0,
			0
		};

		Serializer<UInt_64> payload(Endianness::LE);
		payload.Write(er);
		payload.Resize(payload.Size() + size);

		Util::Copy(&payload[payload.GetOffset()], data, size);

		payload.SetOffset(payload.GetOffset() + size);

		Send(address, header, &payload[0], payload.Size());
	}

	void BaseICMP::SetReceiveTimeout(UInt_64 timeout)
	{
	}

	IP BaseICMP::GetVersion() const
	{
		return version;
	}

	bool BaseICMP::IsValid() const
	{
		return false;
	}

	UInt_16 BaseICMP::ComputeChecksumV4(UInt_16 *buffer, Size length)
	{
		Size sum = 0;
		while (length > 1)
		{
			sum += *buffer++;
			length -= sizeof(UInt_16);
		}

		if (length == 1)
			sum += *(UInt_8 *)buffer;

		sum = (sum >> 16) + (sum & 0xffff);
		sum += (sum >> 16);

		return (UInt_16)~sum;
	}

	UInt_64 BaseICMP::SendV6(const Str_8 &address, ICMP_Header header, const Byte *data, UInt_64 size)
	{
		return 0;
	}

	UInt_64 BaseICMP::SendV4(const Str_8 &address, ICMP_Header header, const Byte *data, UInt_64 size)
	{
		return 0;
	}

	UInt_64 BaseICMP::ReceiveV6(Str_8 &address, ICMP_Header &header, Serializer<UInt_64> &data) const
	{
		return 0;
	}

	UInt_64 BaseICMP::ReceiveV4(Str_8 &address, ICMP_Header &header, Serializer<UInt_64> &data) const
	{
		return 0;
	}
}
